"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const node_utils_1 = require("../node-utils");
const create_testing_library_rule_1 = require("../create-testing-library-rule");
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
exports.RULE_NAME = 'no-debug';
exports.default = create_testing_library_rule_1.createTestingLibraryRule({
    name: exports.RULE_NAME,
    meta: {
        type: 'problem',
        docs: {
            description: 'Disallow unnecessary debug usages in the tests',
            category: 'Best Practices',
            recommended: 'warn',
        },
        messages: {
            noDebug: 'Unexpected debug statement',
        },
        schema: [],
    },
    defaultOptions: [],
    create(context, [], helpers) {
        const suspiciousDebugVariableNames = [];
        const suspiciousReferenceNodes = [];
        const renderWrapperNames = [];
        function detectRenderWrapper(node) {
            const innerFunction = node_utils_1.getInnermostReturningFunction(context, node);
            if (innerFunction) {
                renderWrapperNames.push(node_utils_1.getFunctionName(innerFunction));
            }
        }
        return {
            VariableDeclarator(node) {
                if (!node.init) {
                    return;
                }
                const initIdentifierNode = node_utils_1.getDeepestIdentifierNode(node.init);
                if (!initIdentifierNode) {
                    return;
                }
                const isRenderWrapperVariableDeclarator = initIdentifierNode
                    ? renderWrapperNames.includes(initIdentifierNode.name)
                    : false;
                if (!helpers.isRenderVariableDeclarator(node) &&
                    !isRenderWrapperVariableDeclarator) {
                    return;
                }
                if (node_utils_1.isObjectPattern(node.id)) {
                    for (const property of node.id.properties) {
                        if (node_utils_1.isProperty(property) &&
                            experimental_utils_1.ASTUtils.isIdentifier(property.key) &&
                            property.key.name === 'debug') {
                            const identifierNode = node_utils_1.getDeepestIdentifierNode(property.value);
                            if (identifierNode) {
                                suspiciousDebugVariableNames.push(identifierNode.name);
                            }
                        }
                    }
                }
                if (experimental_utils_1.ASTUtils.isIdentifier(node.id)) {
                    suspiciousReferenceNodes.push(node.id);
                }
            },
            CallExpression(node) {
                const callExpressionIdentifier = node_utils_1.getDeepestIdentifierNode(node);
                if (!callExpressionIdentifier) {
                    return;
                }
                if (helpers.isRenderUtil(callExpressionIdentifier)) {
                    detectRenderWrapper(callExpressionIdentifier);
                }
                const referenceNode = node_utils_1.getReferenceNode(node);
                const referenceIdentifier = node_utils_1.getPropertyIdentifierNode(referenceNode);
                if (!referenceIdentifier) {
                    return;
                }
                const isDebugUtil = helpers.isDebugUtil(callExpressionIdentifier);
                const isDeclaredDebugVariable = suspiciousDebugVariableNames.includes(callExpressionIdentifier.name);
                const isChainedReferenceDebug = suspiciousReferenceNodes.some((suspiciousReferenceIdentifier) => {
                    return (callExpressionIdentifier.name === 'debug' &&
                        suspiciousReferenceIdentifier.name === referenceIdentifier.name);
                });
                if (isDebugUtil || isDeclaredDebugVariable || isChainedReferenceDebug) {
                    context.report({
                        node: callExpressionIdentifier,
                        messageId: 'noDebug',
                    });
                }
            },
        };
    },
});
